"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DecreaseKinesisStreamsShardsTrick = void 0;
const tslib_1 = require("tslib");
const aws_sdk_1 = tslib_1.__importDefault(require("aws-sdk"));
const chalk_1 = tslib_1.__importDefault(require("chalk"));
class DecreaseKinesisStreamsShardsTrick {
    constructor() {
        this.ksClient = new aws_sdk_1.default.Kinesis();
    }
    getMachineName() {
        return DecreaseKinesisStreamsShardsTrick.machineName;
    }
    getDisplayName() {
        return 'Decrease Kinesis Streams Shards';
    }
    canBeConcurrent() {
        return true;
    }
    async conserve(subListr, dryRun) {
        const streamNames = await this.listKinesisStreamsNames();
        const currentState = await this.getCurrentState(streamNames);
        for (const stream of currentState) {
            subListr.add({
                title: chalk_1.default.blueBright(`${stream.name}`),
                task: (ctx, task) => this.conserveStreamShards(task, dryRun, stream),
            });
        }
        return currentState;
    }
    async restore(subListr, dryRun, originalState) {
        for (const table of originalState) {
            subListr.add({
                title: chalk_1.default.blueBright(table.name),
                task: (ctx, task) => this.restoreStreamShards(task, dryRun, table),
            });
        }
    }
    async conserveStreamShards(task, dryRun, streamState) {
        if (dryRun) {
            task.skip('Skipped due to dry-run');
        }
        else if (streamState.state !== 'ACTIVE') {
            task.skip(`State is not Active, it is ${streamState.state} instead.`);
        }
        else if (streamState.shards > 1) {
            let targetShards = streamState.shards;
            let step = 0;
            do {
                targetShards = Math.ceil(targetShards / 2);
                step += 1;
                task.output = `Step #${step}: decreasing shards to ${targetShards}, final target: 1...`;
                await this.ksClient
                    .updateShardCount({
                    StreamName: streamState.name,
                    TargetShardCount: targetShards,
                    ScalingType: 'UNIFORM_SCALING',
                })
                    .promise();
                await this.ksClient
                    .waitFor('streamExists', {
                    StreamName: streamState.name,
                    $waiter: {
                        delay: 15,
                        maxAttempts: 30,
                    },
                })
                    .promise();
            } while (targetShards > 1);
            task.output = `Decreased number of shards to ${targetShards}`;
        }
        else {
            task.skip(`Shards are already at minimum of 1`);
        }
    }
    async restoreStreamShards(task, dryRun, streamState) {
        if (dryRun) {
            task.skip(`Skipped due to dry-run`);
        }
        else if (streamState.shards > 1) {
            const streamSummary = await this.getStreamSummary(streamState.name);
            const currentShards = streamSummary.OpenShardCount;
            if (currentShards >= streamState.shards) {
                task.skip(`Stream shards are already configured to ${currentShards}. Previous number of shards: ${streamState.shards}`);
                return;
            }
            let targetShards = currentShards;
            let step = 0;
            do {
                targetShards = Math.min(Math.floor(targetShards * 2), streamState.shards);
                step += 1;
                task.output = `Step #${step}: increasing shards to ${targetShards}, final target: ${streamState.shards}...`;
                await this.ksClient
                    .updateShardCount({
                    StreamName: streamState.name,
                    TargetShardCount: targetShards,
                    ScalingType: 'UNIFORM_SCALING',
                })
                    .promise();
                await this.ksClient
                    .waitFor('streamExists', {
                    StreamName: streamState.name,
                    $waiter: {
                        delay: 15,
                        maxAttempts: 30,
                    },
                })
                    .promise();
            } while (targetShards < streamState.shards);
        }
        else {
            task.skip(`Shards were already at minimum of 1`);
        }
    }
    async getCurrentState(streamNames) {
        return Promise.all(streamNames.map(async (streamName) => {
            const streamSummary = await this.getStreamSummary(streamName);
            return {
                name: streamName,
                state: streamSummary.StreamStatus,
                shards: streamSummary.OpenShardCount,
            };
        }));
    }
    async listKinesisStreamsNames() {
        return (await this.ksClient.listStreams().promise()).StreamNames || [];
    }
    async getStreamSummary(streamName) {
        return (await this.ksClient
            .describeStreamSummary({
            StreamName: streamName,
        })
            .promise()).StreamDescriptionSummary;
    }
}
exports.DecreaseKinesisStreamsShardsTrick = DecreaseKinesisStreamsShardsTrick;
DecreaseKinesisStreamsShardsTrick.machineName = 'decrease-kinesis-streams-shards';
//# sourceMappingURL=decrease-kinesis-streams-shards.trick.js.map